"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateProcessor = void 0;
const api_typed_1 = require("api-typed");
const child_process_1 = require("child_process");
const handlebars_1 = __importDefault(require("handlebars"));
const path_1 = __importDefault(require("path"));
const typedi_1 = require("typedi");
const SharedTemplate_1 = require("../Templates/SharedTemplate");
const StandardTemplate_1 = require("../Templates/StandardTemplate");
let TemplateProcessor = class TemplateProcessor {
    constructor(sharedTemplate, standardTemplate, logger) {
        this.logger = logger;
        this.templates = [];
        this.templates = [sharedTemplate, standardTemplate];
    }
    getTemplate(name) {
        const template = this.templates.find((t) => t.name === name);
        if (!template) {
            throw new Error(`Could not find template called "${name}"`);
        }
        return template;
    }
    async install(names, projectDir, params) {
        const templateNames = typeof names === 'string' ? [names] : names;
        const templates = templateNames.map((name) => this.getTemplate(name));
        for (const template of templates) {
            await this.installFiles(template, projectDir, params);
        }
        for (const template of templates) {
            await this.installDependencies(template, projectDir);
        }
    }
    async installFiles(template, projectDir, params) {
        this.logger.debug(`Installing files from template "${template.name}"`);
        await Promise.all(template
            .getFiles()
            .map((file) => this.installFile(file, projectDir, params)));
    }
    async installFile(file, projectDir, params) {
        this.logger.debug(`Installing file "${file.projectPath}"`);
        const targetPath = path_1.default.join(projectDir, file.projectPath);
        const fileContent = await api_typed_1.readFile(file.srcPath);
        if (!file.isTemplate) {
            await api_typed_1.writeFile(targetPath, fileContent);
            return;
        }
        const compiled = handlebars_1.default.compile(fileContent)(params);
        await api_typed_1.writeFile(targetPath, compiled);
    }
    async installDependencies(template, projectDir) {
        await this.installPackages(template.getPackages(), projectDir);
        await this.installPackages(template.getDevPackages(), projectDir, {
            dev: true,
        });
    }
    async installPackages(packages, projectDir, { dev = false } = {}) {
        if (packages.length === 0) {
            return;
        }
        const pkgString = packages
            .map((pkg) => {
            if (typeof pkg === 'string') {
                return pkg;
            }
            return `${pkg[0]}@${pkg[1]}`;
        })
            .join(' ');
        const cmd = `npm i ${dev ? '-D' : '-S'} ${pkgString}`;
        this.logger.debug(cmd);
        child_process_1.execSync(cmd, { stdio: 'inherit', cwd: projectDir });
    }
};
TemplateProcessor = __decorate([
    typedi_1.Service(),
    __param(0, typedi_1.Inject(() => SharedTemplate_1.SharedTemplate)),
    __param(1, typedi_1.Inject(() => StandardTemplate_1.StandardTemplate)),
    __param(2, api_typed_1.InjectLogger()),
    __metadata("design:paramtypes", [Object, Object, Object])
], TemplateProcessor);
exports.TemplateProcessor = TemplateProcessor;
//# sourceMappingURL=TemplateProcessor.js.map